/*********************************************************/
/* file: Xcomm.c                                         */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 april 93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  None as yet. if you add place your name here or below*/
/*                                                       */
/* description:                                          */
/*   Xcomm.c handles all the drawing and some of the     */
/* events neccessary for the program                     */
/*********************************************************/
#include <Xcomm.h>
#include <title.h>

/***************************************************/
/* blowup is the error handler for X errors....    */
/* simply prints the the error codes and the code  */
/* of the request that caused it. Adds one to the  */
/* total number of errors, and terminates if too   */
/* many have occurred.                             */
/***************************************************/
int
blowup(Display * display, XErrorEvent * event)
{
  char            data[512];
  fprintf(stderr, "oh no! : X Error # %d \n", ++errnum);
  XGetErrorText(display, event->error_code, data, 500);
  fprintf(stderr, "Error : %s\n", data);
  fprintf(stderr, "\t RQ#: %ld \n", event->serial);
  fprintf(stderr, "\t RQ code: %d\n", event->request_code);
  fprintf(stderr, "\t Error #: %d\n\n", event->error_code);
  if (errnum > errormax)
  {
    fprintf(stderr, "too many errors...\n");
    fprintf(stderr, "Exiting....\n");
    exit(2);
  }
  return (0);

}

/*********************************************/
/* draw_item                                 */
/* draws a single item in a given window at  */
/* the specified x and y coordinates for the */
/* passed player structure                   */
/*********************************************/
void
draw_item(playerstruct * pl, drawdata * win, int x, int y,
    short int item_num, int color)
{
  
  XSetBackground(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 pl->pixels[color<<1]);

  XSetForeground(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 pl->pixels[(color<<1)+1]);

  XSetClipOrigin(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 x,y);

  XCopyPlane(pl->display,pl->masks[picnums[item_num][2]+1],
	     win->window,
	     pl->maskgcs[picnums[item_num][2]],
	     0,0,64,64,x,y,1);
}
void
draw_item_invert(playerstruct * pl, drawdata * win, int x, int y,
    short int item_num, int color)
{
  
  XSetBackground(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 pl->pixels[(color<<1) +1]);

  XSetForeground(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 pl->pixels[color<<1]);

  XSetClipOrigin(pl->display,
		 pl->maskgcs[picnums[item_num][2]],
		 x,y);

  XCopyPlane(pl->display,pl->masks[picnums[item_num][2]+1],
	     win->window,
	     pl->maskgcs[picnums[item_num][2]],
	     0,0,64,64,x,y,1);
}

/*********************************************/
/* these are left over from when there were  */
/* three different versions of draw_item     */
/*********************************************/
void
draw_item2(playerstruct * pl,   drawdata * win, int x, int y,
     short int itemnum)
{
}

void
drawitems()
{
}

/*********************************************/
/* this is resevered for future use as the   */
/* render function for the title screen      */
/*********************************************/
void
render_start(playerstruct * pl)
{
}


/**********************************************/
/* this function will draw the current parser */
/* buffer contents in the lower portion of    */
/* the window                                 */
/**********************************************/ 
void
write_parsing(playerstruct * pl)
{

                  /* j  = first character in the parsing array to write */
                  /* ly = start of parse line in the y coordinate */

  int             ly = pl->th * (pl->tm + 1),
                  i,
                  j = 0;
 
  /* are there > 0 characters on the parsing line? */
  if (pl->px)
  {
    /* i <- the length of the parsing line */
    i = pl->px;
    
    /* check to see if the parsing line is longer than the text window width */
    if (i > pl->tw)
      /* if so then while the length of line is longer than the text window */ 
      while (i > pl->tw)
	{
	  /* draw as many characters as can fit on on line  */
	  XDrawImageString(pl->display, pl->text.window, pl->text.text_gc,
			   2, ly, &pl->parsing[j], pl->tw);
	  
	  /* advance the current y location */
	  ly += pl->th;

	  /* decrement the number of characters drawn */
	  i -= pl->tw;

	  /* advance the starting character index */
	  j += pl->tw;
      }
    
    /* draw what is left over - if the length of the parsing line was */
    /* less than that of the size of the parsing window - then draw   */
    /* that                                                           */
    XDrawImageString(pl->display, pl->text.window, pl->text.text_gc,
         2, ly, &pl->parsing[j], i);
  }
}

/*********************************************/
/* this is a subfunction/subroutine for the  */
/* the write_text function. It will update   */
/* the text information variables in the     */
/* player data extensions, so that text will */
/* be correctly drawn the next time that     */
/* a line of text is written. It updates the */
/* aformentioned data structure to reflect a */
/* single line of text being written to the  */
/* screen. returns 1 if the screen scrolled  */
/*********************************************/
int
update_text_info(playerstruct * pl)
{
  if (pl->tc >= pl->tm) /* is the current location >= the max location? */
  {
    /* if so  */
    /* reset the current line location */
    pl->tc = 0; 
    /* change first line to draw */
    pl->ts = pl->ts >= pl->tm ? 0 : pl->ts + 1;
    /* set scroll mode on */
    pl->sc = 1;
    /* return the fact that we scrolled */
    return 1;
  } else
  {
    /* otherwise ... */
    /* increment the current text line pointer */
    pl->tc++;
    if (pl->sc) /* check for scroll active */
    {
      /* if so... */
      /* reset which line to start drawing from */
      pl->ts = pl->ts >= pl->tm ? 0 : pl->ts + 1;
      /* return fact that we scrolled */
      return 1;
    }
    return (0);
  }
}


/************************************************/
/* the following writes text to the player info */
/* window. The string is broken up (no hyphens) */
/* across lines so that no part is lost. Calls  */
/* update_text_info so that future calls write  */
/* into the text buffer correctly.              */
/************************************************/
void
write_text(playerstruct * pl, char *string)
{
  int             i = 0,
                  /* j is remaing characters to write*/
                  j = strlen(string); 
  
    if (j > pl->tw) /* is string longer than one line? */
      /* if so... */
      while (j > 0) /* perform while there are still characters in buffer */
	{
	  /* copy a piece of the string into the current line of the */
	  /* text buffer                                             */
	  strncpy(pl->info[pl->tc], &string[i], pl->tw);

	  update_text_info(pl);

	  /* terminate the new current line to avoid drawing junk    */
	  pl->info[i][pl->tw] = '\0';

	  /* count the number of characters written for possible */
	  /* future use */
	  i += pl->tw;

	  /* decrement amount of characters in buffer by the number */
	  /* of characters that screen can hold                     */
	  j -= pl->tw;
	}
    else
      { /* otherwise use just copy into the current line            */
	strncpy(pl->info[pl->tc], string, strlen(string));

	/* terminate the new current line to avoid drawing junk     */
	pl->info[pl->tc][strlen(string)] = '\0';

	update_text_info(pl);
      }

  /* reflect fact that text may need to be redrawn*/
  SET_PL_TEXT_WRITTEN(pl);
}


/*************************************************/
/* clear_parsing                                 */
/*   simply blanks out the entire text area      */
/*************************************************/
void
clear_parsing(playerstruct * pl)
{
  XClearArea(pl->display, pl->text.window, 0, pl->th * pl->tm + 2, 0,
       TEXT_H - (pl->th * pl->tm), False);
}

/************************************************/
/* set_pointer:                                 */
/*   set_pointer will determine which icon to   */
/* set the mouse cursor to when it is positioned*/
/* inside of the view window. It uses approx.   */
/* values of tangents to calculate the angle    */
/* between the tip of the pointer and the center*/
/* of the player                                */
/************************************************/
void
set_pointer(playerstruct * pl)
{
  int             rx,
                  ry,
                  wx,
                  wy,
                  num,
                  ratio;
  unsigned int    mr;
  Window          root,
                  child;
  
  /* get the mouse x,y and window */
  XQueryPointer(pl->display, pl->main.window, &root, &child, &rx, &ry,
        &wx, &wy, &mr);

  if(child == pl->view.window) /* check if mouse in the veiw window */
  {
    /* if so..... */
    /* compensate for global coordinates */
    wx -= VIEW_X;
    wy -= VIEW_Y;

    if (wx < PLAYER_X)
      {
	if (wy > PLAYER_Y)
	  {
	    /* calculate the approx. tangent */
	    ratio = (100 * (wy - PLAYER_Y)) / (PLAYER_X - wx);
	    if (ratio < 26) /* just plain left */
	      num = 8;
	    else if (ratio > 175) /* just plain down */
	      num = 6;
	    else
	      num = 7;  /* lower left */
	} else
	  {
 	    /* calculate the approx. tangent */
	    ratio = (100 * (PLAYER_Y - wy)) / (PLAYER_X - wx);
	    if (ratio < 26) /* just plain left */
	      num = 8;
	    else if (ratio > 175) /* just plain up */
	      num = 2;
	    else
	      num = 1;  /* upper left */
	  }
    } else if (wx > PLAYER_X)
      {
	if (wy > PLAYER_Y)
	  {
	    /* calculate the approx. tangent */
	    ratio = (100 * (wy - PLAYER_Y)) / (wx - PLAYER_X);
	    if (ratio < 26) /* just plain right */
	      num = 4;
	    else if (ratio > 175) /* just plain down */
	      num = 6;
	    else
	      num = 5; /* lower right */
	} else
	  {
	    /* calculate the approx. tangent */
	    ratio = (100 * (PLAYER_Y - wy)) / (wx - PLAYER_X);
	    if (ratio < 26) /* just plain right */
	      num = 4;
	    else if (ratio > 175) /* just plain up */
	      num = 2;
	    else
	      num = 3; /* upper right */
      }
    } else /* handle degenerate cases of tangent (90 and 270 degrees)*/
      num = PLAYER_Y > wy ? 2 : 6;  
    
    /* change the cursor if neccessary */
    if (num != (pl->dir - 1))
	XDefineCursor(pl->display, pl->view.window, pl->pointers[num]);
    
    /* store the direction player data structure */
    pl->dir = num - 1;
  }
}


/************************************************/
/* refresh text:                                */
/*   writes all the strings to the player text  */
/* window for a refresh                         */
/************************************************/
void
refresh_text(playerstruct * pl)
{
  int             a,
                  cy = pl->th;
  
  /* clear the area that we will be drawing into */
  XClearArea(pl->display, pl->text.window, 0, 0, 0,
	     pl->th * pl->tm + 1, False);
  
  /* is the current line of text the place to start ? */
  if (pl->tc != pl->ts)
    /* if so.... */
    /* is the screen in scrolling mode ? */
    if (pl->sc) 
      /* if so write text to screen until we wrap around */
      for (a = pl->ts; a != (pl->ts == 0 ? pl->tm : pl->ts - 1);
	   a = (a == pl->tm ? 0 : a + 1))
	{
	  XDrawImageString(pl->display, pl->text.window, pl->text.text_gc,
			   2, cy, pl->info[a], strlen(pl->info[a]));
	  cy += pl->th;
	}
    else
      /* otherwise ...*/
      /* write until we hit the current line of text */
      for (a = pl->ts; a != pl->tc; a++)
	{
	  XDrawImageString(pl->display, pl->text.window, pl->text.text_gc,
			   2, cy, pl->info[a], strlen(pl->info[a]));
	  cy += pl->th;
	}
  /* write the current parse buffer */
  write_parsing(pl);
  
  /* mark the fact that the screen has been redrawn */
  CLEAR_PL_TEXT_WRITTEN(pl);
}
/*******************************************/
/* draw_controls                           */
/*   draws the control menu in the controls*/
/* window.                                 */
/*******************************************/
void
draw_controls (playerstruct *pl)
{	

  /* no comments here - just raw x-output of strings and boxes.. */
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   2,pl->th+2,"MODES",5);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,pl->th+4,CONT_W,pl->th+4);

  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*2,"Interact",8);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*2+2,CONT_W,(pl->th+2)*2+2);

  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*3,"Use",3);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*3+2,CONT_W,(pl->th+2)*3+2);

  /* mark the current active mode in the controls panel */
  if (pl->buttonstate < 3 && pl->buttonstate >0)
    XDrawString(pl->display,pl->controls.window,pl->controls.text_gc,
		2,(pl->th+2)*(pl->buttonstate+1),">",1);
 
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   2,(pl->th+2)*4,"COMMANDS",8);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*4 +2,CONT_W,(pl->th+2)*4+2);
  
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*5,"Container",9);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*5 +2,CONT_W,(pl->th+2)*5+2);
  
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*6,"Examine",7);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*6 +2,CONT_W,(pl->th+2)*6+2);
  
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*7,"Interact",8);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*7 +2,CONT_W,(pl->th+2)*7+2);
  
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*8,"Use Item",8);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*8 +2,CONT_W,(pl->th+2)*8+2);
  
  XDrawImageString(pl->display,pl->controls.window,pl->controls.text_gc,
		   20,(pl->th+2)*9,"Throw Item",8);
  XDrawLine (pl->display,pl->controls.window,pl->controls.text_gc,
	     0,(pl->th+2)*9 +2,CONT_W,(pl->th+2)*9+2);
  
}


/********************************************/
/* refresh_commands:                        */
/*  performs basic operations to redraw the */
/* controls window                          */
/********************************************/
void refresh_commands(playerstruct *pl)
{
	XClearWindow(pl->display,pl->controls.window);
	draw_controls(pl);
	CLEAR_PL_COMS_CHANGED(pl);
}

/*********************************************/
/* draw_stats                                */
/*   draw the stats information into the     */
/* stats window                              */
/*********************************************/
void draw_stats (playerstruct *pl)
{
  int 	bar_len;
  char  tmp[100];

  XDrawRectangle (pl->display,pl->stats.window,pl->stats.text_gc,
		  40,10,106,16);
  XDrawRectangle (pl->display,pl->stats.window,pl->stats.text_gc,
			40,30,106,16);

  /* only draw the status bar if the player is in the positive range */
  if (OB_ATTR(pl->obj).phys > 0 && OB_ATTR (pl->obj).physmax > 0)
    {
      bar_len = 100*OB_ATTR(pl->obj).phys/OB_ATTR(pl->obj).physmax;
      bar_len = (bar_len >100 ? 100:bar_len);
      XClearArea (pl->display,pl->stats.window,43,13,100,10,False);
      XFillRectangle (pl->display,pl->stats.window,pl->stats.text_gc,
		      43,13,bar_len,11);
    }

  /* only draw the status bar if the player is in the positive range */
  if (OB_ATTR(pl->obj).mental > 0 && OB_ATTR (pl->obj).mentmax > 0)
    {	
      bar_len = 100*OB_ATTR(pl->obj).mental/OB_ATTR(pl->obj).mentmax;
      bar_len = (bar_len >100 ? 100:bar_len);
      XClearArea (pl->display,pl->stats.window,43,33,100,10,False);
      XFillRectangle (pl->display,pl->stats.window,pl->stats.text_gc,
		      43,33,bar_len,11);
    }
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    2, 23, "PHYS:",5);
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    2, 43, "MENT:",5);	

  sprintf (tmp,"phys: %4d/%4d   mental:%4d/%4d   level:%3d", 	
	   OB_ATTR(pl->obj).phys,OB_ATTR(pl->obj).physmax,
	   OB_ATTR(pl->obj).mental, OB_ATTR(pl->obj).mentmax,
	   OB_STATS(pl->obj).level);
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    2,60,tmp,strlen(tmp));

  sprintf (tmp,"str:%3d   int:%3d   wis:%3d   dex:%3d   con:%3d",
	   OB_STATS(pl->obj).str,OB_STATS(pl->obj).sma,OB_STATS(pl->obj).wis,
	   OB_STATS(pl->obj).dex,OB_STATS(pl->obj).con);
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    2,60+pl->th,tmp,strlen(tmp));

  sprintf (tmp,"luck:%3d   looks:%3d   cha:%3d   damage:%3d",
	   OB_STATS(pl->obj).luck, OB_STATS(pl->obj).looks,
	   OB_STATS(pl->obj).cha, OB_STATS(pl->obj).damage);
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    2,60+2*pl->th,tmp,strlen(tmp));
  
  XClearArea(pl->display,pl->stats.window,  
	     160,20,200,30,False);
  
  sprintf(tmp,"%s the %s",GET_NAME(pl->obj),race_names[pl->obj->subtype]);
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    160,30,tmp,strlen(tmp));
  
  if (AN_OB_ITEM(pl->obj).held == NULL)
    sprintf (tmp,"Holding: NOTHING");
  else
    sprintf (tmp,"Holding: %s",GET_NAME(AN_OB_ITEM(pl->obj).held));
  
  XDrawImageString (pl->display,pl->stats.window,pl->stats.text_gc,
		    160,30+pl->th,tmp,strlen(tmp));
}


/*********************************************/
/* refresh_stats                             */
/*   performs basic commands to redaw the    */
/* stats window                              */
/*********************************************/
void refresh_stats (playerstruct *pl)
{
 XClearArea (pl->display,pl->stats.window,40,10,50,110,False);
 draw_stats(pl);
 CLEAR_PL_STATS_CHANGED (pl);
}

/*******************************************/
/* draw_items                              */
/* updates the player item window          */
/*******************************************/
void 
draw_items (playerstruct *pl)
{
  object  * tmp;
  int       cy,
            cx,
            num_vert;

  /* draw the basic commands at the top of the window */
  XDrawImageString (pl->display,pl->item.window,pl->item.text_gc,
		    5,pl->th,"Shift Up",8);
  
  XDrawImageString (pl->display,pl->item.window,pl->item.text_gc,
		    ITEM_W/4+5,pl->th,"Previous",8);
  
  XDrawImageString (pl->display,pl->item.window,pl->item.text_gc,
		    ITEM_W/2+5,pl->th,"Next",4);
  
  XDrawImageString (pl->display,pl->item.window,pl->item.text_gc,
		    3*ITEM_W/4+5,pl->th,"Shift Down",10);
  
  /* draw lines to partition the commands */
  XDrawLine(pl->display,pl->item.window,pl->item.text_gc,
	    0,pl->th+2,ITEM_W,pl->th+2);
  
  XDrawLine(pl->display,pl->item.window,pl->item.text_gc,
	    ITEM_W/4,0,ITEM_W/4,pl->th+2);
  
  XDrawLine(pl->display,pl->item.window,pl->item.text_gc,
	    ITEM_W/2,0,ITEM_W/2,ITEM_H);
  
  XDrawLine(pl->display,pl->item.window,pl->item.text_gc,
	    3*ITEM_W/4,0,3*ITEM_W/4,pl->th+2);
 
  /* calculate the number of items that can be held vertically */
  num_vert = (ITEM_H - pl->th -2)/((ITEM_H - pl->th -2)/64);
 
  /* try to draw four columns of objects */
  for (tmp = pl->top_held,cx = ITEM_START_X; cx < ITEM_W; cx += ITEM_W/4)
    /* draw four items vertically or until there are no more to draw */
    for (cy = ITEM_START_Y; (cy+64) < ITEM_H && tmp !=NULL;
	 tmp = tmp->below,cy+= num_vert)
      {
	/* actually draw  the item */
	  draw_item (pl,&pl->item,cx,cy,tmp->picnum,tmp->color);
	/* draw a lable on the item */
	XDrawImageString (pl->display,pl->item.window,pl->item.text_gc,
			  cx + ITEM_TEXT_X,cy+32,tmp->name,strlen(tmp->name));

	/* draw a rectangle around the item */
	XDrawRectangle(pl->display,pl->item.window,pl->item.text_gc,
		       cx - ITEM_START_X,cy,ITEM_W/4,num_vert);
	
	/* if the item is equipped highlight it*/
	if (IS_EQUIPPED(tmp)) 
	  /* draw a rectangle in XOR mode to invert the colors */
	  XFillRectangle (pl->display,pl->item.window,pl->item.itemxor_gc,
			cx - ITEM_START_X,cy,ITEM_W/4,num_vert);
	
      }
  
}

/*******************************************/
/* refresh_item                            */
/*   performs basic operations to refresh  */
/* the item window                         */
/*******************************************/
void 
refresh_item (playerstruct *pl)
{
  XClearWindow (pl->display,pl->item.window);
  draw_items (pl);
  CLEAR_PL_ITEM_CHANGED(pl);
}
  

/*******************************************/
/* refresh_view                            */
/*   refresh the view window by copying the*/
/* backing pixmap that contains the actual */
/* picture. this may be changed in the     */
/* future for color                        */
/*******************************************/  
void
refresh_view(playerstruct * pl)
{
  
  XCopyArea(pl->display, pl->viewpix.window, pl->view.window,
      pl->view.itemfg_gc, 0, 0, VIEW_W, VIEW_H, 0,0);
}



/********************************************/
/* RENDERING FUNCTONS%%%%%%%%%%%%%%%%%%%%%%%*/
/* this is one few truly complex routines   */
/* in the program - It is ~VERY~ easy to get*/
/* lost inside this function - if you have  */
/* no wish or need to examine the rendering */
/* functions READ NO FURTHER - this stuff   */
/* can be difficult to keep track of        */
/********************************************/



/********************************************/
/* render_floor                             */
/*   this function will calculate a string  */
/* of objects that are on the "floor" in    */
/* addition to calculating the length of the*/
/* string so that it may be passed to X     */
/* drawing functions. it also calculates how*/
/* far in in the x direction we need to     */
/* start the drawing. I considered breaking */
/* it up into smaller parts but - drawing is*/
/* the most expensive part of the program   */
/* and I wished not to incur any further    */
/* function call overhead                   */
/********************************************/
int
render_floor(playerstruct * pl, int mapx, int mapy, short int bgstring[12],
       short int fgstring[12],short int clstring[12], int *xdelta, int line)
{
  int             tmpx,
                  tmpy,
                  tdir,
                  count,
                  change = 0,
                  ref = pl->obj->facing/2,
                  iter;
  
  /* make an initial calculation of how far in the x direction we start */
  /* the formula below forms a diamond for the initial guess based on   */
  /* the line number                                                    */
  *xdelta = abs (224 - (line*32));

  
  for (tmpx = mapx, tmpy = mapy, count = 0, change = 0, iter = 0;
       iter < (7-(abs(7-line))) ;
       tmpx+= rit[ref][0], tmpy-= rit[ref][1],iter++)
    {
      if (IN_MAP(pl->obj->mapin, tmpx, tmpy) &&
	  HAS_FLOOR(pl->obj->mapin, tmpx, tmpy) &&
	  pl->in_sight[tmpx-pl->lowx][tmpy-pl->lowy])
	{
	  tdir = (HEAD(MAP_FLOOR_OBJ(pl->obj->mapin,tmpx,tmpy))->facing)/2;
	  tdir = (4 +(tdir - ref))%4;
	
	  fgstring[count] = bgstring[count] =
	    (short)
	      picnums[MAP_FLOOR_OBJ(pl->obj->mapin,tmpx,tmpy)->picnum][tdir];
	  
	  clstring[count] =
	    (MAP_FLOOR_OBJ(pl->obj->mapin,tmpx,tmpy)->color)<<1;

	  fgstring[count]++;
	  count ++;
	  
	  change += count - change;
      
      } else
	{
	  if (change)
	    {
	      fgstring[count] = bgstring[count] 
		= EMPTYSQUARE;
	      clstring[count] = 0;
	      fgstring[count]++;
	      count++;
	  } else
	    *xdelta += 64;
	}
    }
  return (change);
}

object *
nugget(object *o)
{ 
	fprintf (stderr,"--%s  %s %d %d --|",GET_NAME(o),GET_NAME(o->holding),o->facing,o->holding->facing);
	return o->holding;
}

int
render_items(playerstruct * pl, int mapx, int mapy, int mapz, short int
	     bgstring[12], short int fgstring[12], short int clstring[12],
	     int *xdelta, int line)
{
  int             tmpx,
                  tmpy,
                  tdir,
                  count,
                  change = 0,
                  ref = pl->obj->facing/2,
                  iter;
  object *tobj;
  
  *xdelta = abs (224 - (line*32));
  for (tmpx = mapx, tmpy = mapy, count = 0, change = 0,iter = 0;
       iter < (7-(abs(7-line)));
       tmpx+=rit[ref][0],tmpy-=rit[ref][1],iter++)
    {
      if (IN_MAP(pl->obj->mapin, tmpx, tmpy) &&
	  HAS_ITEM(pl->obj->mapin, tmpx, tmpy, mapz) &&
	  pl->in_sight[tmpx - pl->lowx][tmpy-pl->lowy])
	{
	  if (pl->in_sight[tmpx -pl->lowx][tmpy-pl->lowy] == 2)
	    {
	      fgstring[count] = bgstring[count] = 71;
	      clstring [count] = 0;
	    }
	  else
	    {
	      tobj = TOP_MAP_OBJ(pl->obj->mapin,tmpx,tmpy,mapz);
	      tdir = (HEAD(tobj)->facing)/2;
	      
	      tdir = (4 +(tdir - ref))%4;
	      fgstring[count] = bgstring[count] 
		= (short) picnums[tobj->picnum][tdir];
	      clstring[count] = (HEAD(tobj)->color)<<1;
	      
	    }
	  
	  
	  fgstring[count]++;count++;
	  
	  change += count - change;
      
      } else
	{
	  if (change)
	    {
	      fgstring[count] = bgstring[count] 
		= EMPTYSQUARE;
	      clstring[count] = 0;
	      fgstring[count]++;
	      count++;
	  
	  } else
	  
	    *xdelta += 64;
	}
    }
  return (change);
}

void
render2(playerstruct * pl)
{
  short int       bg_string[4][23][12],
                  fg_string[4][23][12];
  short int        color_string[4][23][12];
  int             cur_x,
                  cur_y,
                  map_x,
                  map_y,
                  rend_x,
                  rend_y,
                  zbuf,
                  lengths[4][23],
                  xstarts[4][23],
                  num = 0,
                  ref = pl->obj->facing/2,
                  iter;
  int tvar,tx;
                  
  for (map_y = pl->obj->y+(NUM_TILES*tvecs[ref][1]+(MAX_X_TILES*tvecs[ref][0])),
       map_x = pl->obj->x-(MAX_X_TILES*tvecs[ref][1])
       +(NUM_TILES*tvecs[ref][0]),
       iter = 1;
       iter < 14;
       map_x -= (iter>6?tvecs[ref][0]:tvecs[ref][1]),
       map_y += (iter>6?-tvecs[ref][1]:tvecs[ref][0])
       ,iter++)
    {
      lengths[3][iter] = render_floor(pl, map_x, map_y, bg_string[3][iter],
				    fg_string[3][iter],color_string[3][iter],
				    &(xstarts[3][iter]),iter);
      for (zbuf = 0; zbuf < 3; zbuf++)
	{
	  lengths[zbuf][iter] = render_items(pl, map_x, map_y, zbuf,
					     bg_string[zbuf][iter],
					     fg_string[zbuf][iter],
					     color_string[zbuf][iter],
					     &(xstarts[zbuf][iter]), iter);
	}
    }
  cur_y = START_RENDER_Y;
  cur_x = START_RENDER_X;

  XFillRectangle(pl->display, pl->viewpix.window,
           pl->viewpix.text_gc, 0, 0, VIEW_W, VIEW_H);
  num = 0;
  rend_y = cur_y;
  for (num = 1; num < 14; num++)
  {
    rend_x = cur_x;
    rend_y = cur_y;
   
    if (lengths[3][num])
    {
      for (tvar = 0,tx = xstarts[3][num]; tvar < lengths[3][num]; tvar++, tx+=64)
	if (bg_string[3][num][tvar]!=EMPTYSQUARE)
	  {
	    XSetBackground(pl->display,
			   pl->maskgcs[bg_string[3][num][tvar]],
			   pl->pixels[color_string[3][num][tvar]]);
	    XSetForeground(pl->display,
			   pl->maskgcs[bg_string[3][num][tvar]],
			   pl->pixels[color_string[3][num][tvar]+1]);
	    XSetClipOrigin(pl->display,
			   pl->maskgcs[bg_string[3][num][tvar]],
			   tx,rend_y);
	    XCopyPlane(pl->display,pl->masks[bg_string[3][num][tvar]+1],
		       pl->viewpix.window,
		       pl->maskgcs[bg_string[3][num][tvar]],
		       0,0,64,64,tx,rend_y,1);
	  }
    
    }
 
    for (zbuf = 0; zbuf < 3; zbuf++)
    {
      if (lengths[zbuf][num])
	{
	  for (tvar = 0,tx = xstarts[zbuf][num]; 
	       tvar < lengths[zbuf][num]; tvar++, tx+=64)
	    if (bg_string[zbuf][num][tvar]!=EMPTYSQUARE)
	      {
		XSetBackground(pl->display,
			       pl->maskgcs[bg_string[zbuf][num][tvar]],
			       pl->pixels[color_string[zbuf][num][tvar]]);
		XSetForeground(pl->display,
			       pl->maskgcs[bg_string[zbuf][num][tvar]],
			       pl->pixels[color_string[zbuf][num][tvar]+1]);
		XSetClipOrigin(pl->display,
			       pl->maskgcs[bg_string[zbuf][num][tvar]],
			       tx,rend_y);
		
		XCopyPlane(pl->display,pl->masks[bg_string[zbuf][num][tvar]+1],
			   pl->viewpix.window,
			   pl->maskgcs[bg_string[zbuf][num][tvar]],
			   0,0,64,64,tx,rend_y,1);
		
	      }
	
	}
      rend_y -= VERTTRANS;
    }
    cur_y += TILE_Y_OFFSET;
  }
}

/********************************************/
/* reserved for future use....              */
/*   I'll slap a picture as a title screen  */
/* on if somebody can come up with a decent */
/* one in x bit map format                  */
/********************************************/
void
testdraw(playerstruct *pl)
{

  Pixmap titlepix;

  titlepix = XCreatePixmapFromBitmapData(pl->display,pl->item.window,title_bits,200,200,pl->blackpix,pl->whitepix,1);
  
  XCopyPlane(pl->display,titlepix,pl->item.window,pl->main.itemfg_gc,0,0,200,200,0,0,1);

}




