/*********************************************************/
/* file: Xstart.c                                        */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 april 93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  if you add place your name here or below             */
/*                                                       */
/* description:                                          */
/*   Xstart.c contains all the subroutine neccessary to  */
/* get connected to an Xserver and to initialize data    */
/* structures and parameters neccessary for the operation*/
/* of the program                                        */
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <Xstart.h>

/*****************************************************/
/* init_gc_values                                    */
/* returns a filled in XGCValues structure based on  */
/* the arguments supplied.                           */
/*      bg = background pixel color                  */
/*      fg = foreground pixel color                  */
/*      fnt = font id for this GC                    */
/*      func = X gc function to apply                */
/*****************************************************/
XGCValues
* init_gc_values(unsigned long fg, unsigned long bg, Font fnt, int func)
{
  static XGCValues      gc_values;
  
  gc_values.background = bg;
  gc_values.foreground = fg;
  gc_values.function = func;
  gc_values.font = fnt;
  return (& gc_values); 
}

/****************************************************/
/* set_wm_hints                                     */
/*    sets the icon, and various other handy things */
/*    for the window manager.                       */
/****************************************************/
void
set_wm_icon (playerstruct *pl)
{
  Pixmap    grudge_icon,
            grudge_mask;

  XWMHints  icon_hint ;

  GC        temp_gc,temp_gc2;
  char   icon_string[2];

  icon_string[0] = 1;

  icon_string[1] = '\0';

  grudge_icon = XCreatePixmap(pl->display,pl->main.window,
			      64,64,1);

  grudge_mask = XCreatePixmap(pl->display,pl->main.window,
			      64,64,1);
  
  temp_gc = XCreateGC(pl->display,grudge_icon,GC_MASK,
		      init_gc_values(pl->blackpix, pl->whitepix,
				     pl->itemfont->fid, GXcopy));

  temp_gc2 = XCreateGC(pl->display,grudge_icon,GC_MASK,
		       init_gc_values(pl->whitepix, pl->blackpix,
				      pl->itemfont->fid, GXcopy));

  XFillRectangle (pl->display,grudge_icon,temp_gc2,0,0,64,64);
  
  XDrawImageString(pl->display,grudge_icon,temp_gc,0,64,icon_string,1);

  icon_hint.icon_pixmap  = grudge_icon;
  icon_hint.icon_mask = grudge_mask;
  icon_hint.flags = IconPixmapHint;

  XSetWMHints (pl->display,pl->main.window,&icon_hint);

  XFreeGC(pl->display,temp_gc);
  XFreeGC(pl->display,temp_gc2);
}
  
 
/*****************************************************/
/* init_size_hints                                   */
/* returns a filled in XSizeHints structure based on */
/* the arguments supplied.                           */
/*     hgt = window hieght                           */
/*     wid = window width                            */
/*     bght = base hieght                            */
/*     bwid = base width                             */
/*     filled = what fields are filled in            */
/*****************************************************/
XSizeHints
* init_size_hints(int wid, int hgt, int bwid, int bhgt, long filled)
{
 static XSizeHints     size_hints;
/*  size_hints = (XSizeHints *) malloc(sizeof(XSizeHints)); */
  size_hints.x = 0;
  size_hints.y = 0;
  size_hints.height = hgt;
  size_hints.width = wid;
  size_hints.min_width = wid;
  size_hints.min_height = hgt;
  size_hints.max_width = wid;
  size_hints.max_height = hgt;
  size_hints.base_height = bhgt;
  size_hints.base_width = bwid;
  size_hints.flags = filled;
  return ( & size_hints);
}

/***************************************************/
/* init_class_hint                                 */
/* mallocs and fills in an XClassHint hint         */
/* structure                                       */
/*    *class: pointer to string defining app. class*/
/*    *name: pointer to string defining app. name  */
/***************************************************/
XClassHint
* init_class_hint(char *class, char *name)
{
 static  XClassHint     class_hints;
/*  class_hints = (XClassHint *) malloc(sizeof(XClassHint));*/
  class_hints.res_class = class;
  class_hints.res_name = name;
  return (&class_hints);
}


/****************************************************/
/* init_wm_hints                                    */
/* mallocs, fills in and returns an XWMHints        */
/* structure                                        */
/*    mask = what fields are filled in              */
/*    initial = initial application state           */
/*    inp = input typification                      */
/****************************************************/
XWMHints
* init_wm_hints(long mask, int initial, Bool inp)
{
  static XWMHints       wm_hints;
/*  wm_hints = (XWMHints *) malloc(sizeof(XWMHints));*/
  wm_hints.flags = mask;
  wm_hints.initial_state = initial;
  wm_hints.input = inp;
  return (& wm_hints);
}

/****************************************************/
/* init_win_attr                                    */
/* mallocs, fills in and returns an                 */
/* XSetWindowAttributes structure                   */
/****************************************************/
XSetWindowAttributes
* init_win_attr(long events, unsigned long bp, unsigned long wp)
{
 static XSetWindowAttributes win_attr;
/*  win_attr = (XSetWindowAttributes *) malloc(sizeof
               (XSetWindowAttributes));*/
  win_attr.event_mask = events;
  win_attr.border_pixel = bp;
  win_attr.background_pixel = wp;
  return (& win_attr);
}

/******************************************************/
/* init_main_attributes                               */
/* mallocs, fills in and returns an                   */
/* XSetWindowAttributes structure for the main window */
/* has same arguments as init_win_attr but also       */
/* includes room for background pixmap                */
/******************************************************/
XSetWindowAttributes
* init_main_attr(long events, unsigned long bp,
     unsigned long wp,  playerstruct * pl)
{
 static  XSetWindowAttributes win_attr;
/* win_attr = (XSetWindowAttributes *) malloc(sizeof
               (XSetWindowAttributes));*/
  win_attr.event_mask = events;
  win_attr.border_pixel = bp;
  win_attr.background_pixmap = None;
  return (&win_attr);
}

/*****************************************************/
/* set_back_pixmap                                   */
/* sets the background pixmap for a given window     */
/* reads it in from the supplied filename of a bitmap*/
/*     bitmap = pointer to name of bitmap file       */
/*     pl = pointer to playerstruct data             */
/*     win_data = pointer to window structure for    */
/*                which to set the background        */
/*****************************************************/
void
set_back_pixmap(playerstruct * pl,  drawdata * win_data, char *bitmap)
{
  Pixmap          pixmap1, pixmap2;
  int             d;
  int             status,
                  x,
                  y;
  unsigned int    xh,
                  yh;
  char            path[128];
  
  strcpy(path, LIBDIR);
  strcat(path, "/");
  strcat(path, bitmap);
  status = XReadBitmapFile(pl->display, win_data->window, path,
         &xh, &yh, &pixmap1, &x, &y);
  switch (status)
  {
  case (Success):
    break;
  case (BitmapOpenFailed):
    fprintf(stderr, "bitmap open failed");
    exit(2);
    break;
  default:
    fprintf(stderr, "some kind of error");
    exit(3);
    break;
  }
  d = DefaultDepth(pl->display,DefaultScreen(pl->display));
  if (d > 1)
    {
      pixmap2 =  XCreatePixmap(pl->display, pl->main.window,64,64,d);
      XCopyPlane(pl->display, pixmap1, pixmap2,
		 pl->main.itemfg_gc, 0, 0, 64, 64, 0,0,1);
      XSetWindowBackgroundPixmap(pl->display, win_data->window, pixmap2);
    }
  else
      XSetWindowBackgroundPixmap(pl->display, win_data->window, pixmap1);

}

/**************************************************/
/* set_gc                                         */
/* set the GC structures for a given window or    */
/* pixmap.                                        */
/*      pl = pointer to player data structure     */
/*      win_data = pointer to particular window in*/
/*                 player data structure          */
/**************************************************/
void
set_gc( playerstruct * pl, drawdata * win_data)
{
  win_data->itembg_gc = XCreateGC(pl->display, win_data->window, GC_MASK,
         init_gc_values(pl->whitepix, pl->blackpix, pl->itemfont->fid,
            GXcopy));
  win_data->itemfg_gc = XCreateGC(pl->display, win_data->window, GC_MASK,
         init_gc_values(pl->blackpix, pl->whitepix, pl->itemfont->fid,
            GXcopy));
  win_data->itemxor_gc = XCreateGC(pl->display, win_data->window, GC_MASK,
         init_gc_values(1, 0, pl->itemfont->fid,
            GXxor));
  win_data->text_gc = XCreateGC(pl->display, win_data->window, GC_MASK,
         init_gc_values(pl->blackpix, pl->whitepix, pl->textfont->fid,
            GXcopy));

}

/***************************************************/
/* init_display                                    */
/* opens a connection to the X server at the       */
/* display specified in the supplied player data   */
/*      pl = pointer to player data structure      */
/***************************************************/
int
init_display(playerstruct * pl)
{
  int             screen;
  if ((pl->display = XOpenDisplay(pl->display_name)) == NULL)
  {
    fprintf(stderr, "could not open %s \n", pl->display_name);
    return 1;
  }
  screen = DefaultScreen(pl->display);
  pl->blackpix = BlackPixel(pl->display, screen);
  pl->whitepix = WhitePixel(pl->display, screen);
  pl->root = RootWindow(pl->display, screen);
  return 0;
}

/***************************************************/
/* make_view_window                                */
/* create the viewing window in the main window    */
/* for the supplied player data                    */
/*     pl = pointer to player data structure       */
/***************************************************/
void
make_view_window(playerstruct * pl)
{
  pl->view.window = XCreateWindow(pl->display, pl->main.window, VIEW_X, VIEW_Y,
        VIEW_W, VIEW_H, 2, CopyFromParent, InputOutput, CopyFromParent,
       VAL_MASK, init_win_attr(WIN_EVENTS, pl->blackpix, None));
  set_gc(pl, &pl->view);
  pl->view.size_hint = init_size_hints(VIEW_W, VIEW_H, VIEW_W, VIEW_H,
               SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->view.window, pl->view.size_hint);
  XSetWMHints(pl->display, pl->view.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->view.window, init_class_hint(CLASS, VIEWNAME));
}

/***************************************************/
/* make_main_window                                */
/* creates the main window in the root window for  */
/* the supplied player data                        */
/*     pl = pointer to player data structure       */
/***************************************************/
void
make_main_window(playerstruct * pl)
{
  pl->main.window = XCreateWindow(pl->display, pl->root, MAIN_X, MAIN_Y,
        MAIN_W, MAIN_H, 2, CopyFromParent, InputOutput, CopyFromParent,
          MAIN_MASK, init_main_attr(MAIN_EVENTS, pl->blackpix, pl->whitepix, pl));
  set_gc(pl, &pl->main);
  pl->main.size_hint = init_size_hints(MAIN_W, MAIN_H, MAIN_W, MAIN_H,
               SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->main.window, pl->main.size_hint);
  XSetWMHints(pl->display, pl->main.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->main.window, init_class_hint(CLASS, MAINNAME));
  XStoreName(pl->display, pl->main.window, TITLE);

  
}

/****************************************************/
/* make_text_window                                 */
/* makes the text window in the main window for the */
/* supplied player data                             */
/*     pl = pointer to player data structure        */
/****************************************************/
void
make_text_window(playerstruct * pl)
{
  pl->text.window = XCreateWindow(pl->display, pl->main.window, TEXT_X, TEXT_Y,
    TEXT_W, TEXT_H, 2, CopyFromParent, InputOutput, CopyFromParent, VAL_MASK,
         init_win_attr(WIN_EVENTS, pl->blackpix, pl->whitepix));
  set_gc(pl, &pl->text);
  pl->text.size_hint = init_size_hints(TEXT_W, TEXT_H, TEXT_W, TEXT_H,
               SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->text.window, pl->text.size_hint);
  XSetWMHints(pl->display, pl->text.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->text.window, init_class_hint(CLASS, TEXTNAME));
}

/*****************************************************/
/* make_item_window                                  */
/* creates the item window in the main window for the*/
/* supplied player data                              */
/*     pl = pointer to player data structure        */
/*****************************************************/
void
make_item_window(playerstruct * pl)
{
  pl->item.window = XCreateWindow(pl->display, pl->main.window, ITEM_X, ITEM_Y,
        ITEM_W, ITEM_H, 2, CopyFromParent, InputOutput, CopyFromParent,
     VAL_MASK, init_win_attr(WIN_EVENTS, pl->blackpix, pl->whitepix));
  set_gc(pl, &pl->item);
  pl->item.size_hint = init_size_hints(ITEM_W, ITEM_H, ITEM_W, ITEM_H,
               SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->item.window, pl->item.size_hint);
  XSetWMHints(pl->display, pl->item.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->item.window, init_class_hint(CLASS, ITEMNAME));
}

/******************************************************/
/* make_control_window                                */
/* creates the controls window in the main window for */
/* the supplied player data                           */
/*     pl = pointer to player data structure          */
/******************************************************/

void
make_control_window(playerstruct * pl)
{
  pl->controls.window = XCreateWindow(pl->display, pl->main.window, CONT_X,
      CONT_Y, CONT_W, CONT_H, 2, CopyFromParent, InputOutput, CopyFromParent,
     VAL_MASK, init_win_attr(WIN_EVENTS, pl->blackpix, pl->whitepix));
  set_gc(pl, &pl->controls);
  pl->controls.size_hint = init_size_hints(CONT_W, CONT_H, CONT_W, CONT_H,
             SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->controls.window, pl->controls.size_hint);
  XSetWMHints(pl->display, pl->controls.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->controls.window,
          init_class_hint(CLASS, CONTNAME));
}

/*******************************************************/
/* make_stats_window                                   */
/* creates the statistics window in the main window for*/
/* the supplied player data structure                  */
/*     pl = pointer to player data structure           */
/*******************************************************/
void
make_stats_window(playerstruct * pl)
{
  pl->stats.window = XCreateWindow(pl->display, pl->main.window, STAT_X,
      STAT_Y, STAT_W, STAT_H, 2, CopyFromParent, InputOutput, CopyFromParent,
     VAL_MASK, init_win_attr(WIN_EVENTS, pl->blackpix, pl->whitepix));
  set_gc(pl, &pl->stats);
  pl->stats.size_hint = init_size_hints(STAT_W, STAT_H, STAT_W, STAT_H,
                SIZE_HINT_MASK);
  XSetWMNormalHints(pl->display, pl->stats.window, pl->stats.size_hint);
  XSetWMHints(pl->display, pl->stats.window,
        init_wm_hints(WM_MASK, NormalState, True));
  XSetClassHint(pl->display, pl->stats.window,
          init_class_hint(CLASS, CONTNAME));
}

/***********************************************/
/* make_viewpix                                */
/* sets up a buffer pixmap for the view window */
/* fills in its gc's also                      */
/***********************************************/
void
make_viewpix(playerstruct * pl)
{
  pl->viewpix.window = XCreatePixmap(pl->display, pl->main.window, VIEW_W,
             VIEW_H, DefaultDepth(pl->display,DefaultScreen(pl->display)));
  set_gc(pl, &pl->viewpix);
}

/***********************************************/
/* make_itempix                                */
/* sets up a buffer pixmap for the item window */
/* fills in its gc's also                      */
/***********************************************/
void
make_itempix(playerstruct * pl)
{
  pl->itempix.window = XCreatePixmap(pl->display, pl->main.window, ITEM_W,
             ITEM_H, 1);
  set_gc(pl, &pl->itempix);
}

/***************************************************/
/* make_cursors                                    */
/* makes cursors in the X server for the players   */
/* display. cursors are created from the icon font */
/* using XCreateGlyphCursor.                       */
/***************************************************/
void
make_cursors(playerstruct * pl)
{
  int             screen,
  								count;
  XColor          fg,
                  bg,
                  tfg,
                  tbg;
  Colormap        tmap;
  
  XWindowAttributes winattr;
  
 
  screen = DefaultScreen(pl->display);
  
  XGetWindowAttributes (pl->display,pl->view.window, &winattr);
  
  tmap = winattr.colormap;
        
  XLookupColor(pl->display,tmap,"Black",&tfg,&fg);
  XLookupColor(pl->display,tmap,"White",&tbg,&bg);

  
  for (count = 0; count < CURSORMAX; count++)
  {
    pl->pointers[count] = XCreateGlyphCursor(pl->display, pl->cursors->fid,
      pl->cursors->fid, (2*count+1), (2 * count), &fg, &bg); 
  }
  
 }

/**************************************************/
/* load_font                                      */
/* loads the font into the server for the players */
/* display. exit if font not found                */
/**************************************************/
XFontStruct *
load_font(playerstruct * pl, char *fontname, XFontStruct ** font)
{
  *font = XLoadQueryFont(pl->display, fontname);
  
  return *font;
}

/*******************************************/
/* load_fonts                              */
/*   run through the list of fonts, and    */
/* attempt to load each one. attempt to    */
/* load a backup font for the text font if */
/* it failed to load                       */
/*******************************************/
void 
load_fonts(playerstruct *pl)
{
  
  char *tmp;
  
  if (load_font(pl, ITEMFONT, &pl->itemfont) == NULL)
    {
      fprintf(stderr,
	      "font %s,used for drawing object icons NOT found ... exiting\n",
	      ITEMFONT);
      exit(2);
    }
  
  if(load_font(pl, CURSORS, &pl->cursors) == NULL)
    {
      fprintf(stderr, "font %s, used for cursors NOT found ... exiting\n",
	      CURSORS);
      exit(2);
    }
  
  if ((tmp=XGetDefault(pl->display,RES_NAME,"textfont"))!=NULL)
    {
      if (load_font(pl, tmp, &pl->textfont) != NULL)
	return;
      else 
	fprintf(stderr,"Player specified font not found trying font %s\n",
		TEXTFONT);
    }
  
  if(load_font(pl, TEXTFONT,&pl->textfont) == NULL)
    {
      fprintf(stderr, "font %s,used for text NOT found ... exiting\n",
	      TEXTFONT);
      exit(2);
    }
}	

/******************************************/
/* alloc_colors                           */
/*   attempt to allocate colors from the  */
/* color map specified by cm, noramlly the*/
/* default/shared colormap                */
/******************************************/
int alloc_colors(playerstruct *pl, Colormap cm)
{
  int index;
  int status;

  XColor screendef,exactdef;

  for (index = 0; index < NUM_COLORS; index ++)
    {
      status = XAllocNamedColor(pl->display, cm,
				background_color_names[index],
				&screendef,&exactdef);

      pl->pixels[index << 1] = screendef.pixel;

      if (status == 0 || status == BadColor || status == BadAccess
	  || status == BadValue || status == BadName)
	{
	  fprintf (stderr,"could not allocate background color #%d - %s\n",
		   index,background_color_names[index]);
	  return 0;
	}
      
      status = XAllocNamedColor(pl->display,cm,
				foreground_color_names[index],
				&screendef,&exactdef);

      pl->pixels[(index << 1)+1] = screendef.pixel;

      if (status == 0 || status == BadColor || status == BadAccess
	  || status == BadValue || status == BadName)
	{
	   fprintf (stderr,"could not allocate foreground color #%d - %s\n",
		   index,foreground_color_names[index]);
	  return 0;
	}
    }	  
  return 1;
}

/*********************************************/
/* store_colors                              */
/*   attempt to store colors in the colormap */
/* specified by cm. upon success set the     */
/* main windows colormap to the one passed to*/
/* the function. this is used when we can no */
/* longer allocate colors from the default/  */
/* shared colormap                           */
/*********************************************/
int store_colors(playerstruct *pl, Colormap cm)
{
  int index;
  int status;

  for (index = 0; index < NUM_COLORS; index ++)
    {
      status = XStoreNamedColor(pl->display, cm,
				background_color_names[index],
				index<<1, FLAGS);

      pl->pixels[index << 1] = index <<1;

      if (status == 0 || status == BadColor || status == BadAccess
	  || status == BadValue || status == BadName)
	{
	 fprintf (stderr,"could not store background color #%d - %s\n",
		   index,background_color_names[index]); 
	  return 0;
	}
            
      status = XStoreNamedColor(pl->display,cm,
				foreground_color_names[index],
				(index << 1)+1,FLAGS);

      pl->pixels[(index << 1)+1] = (index<<1)+1;

      if (status == 0 || status == BadColor || status == BadAccess
	  || status == BadValue || status == BadName)
	{
	  fprintf (stderr,"bad allocation2\n");
	  return 0;
	}
     
    }	
  /* upon success set the main windows colormap to the new one */
  XSetWindowColormap(pl->display, pl->main.window, cm);
  fprintf (stderr,"success\n");
  return 1;
}

/***************************************************/
/* create gcs                                      */
/*   will attempt to allocate the neccessary gcs   */
/* in the colormap if the depth of the display is  */
/* sufficient, otherwise will create a new colormap*/
/* private to grudge                               */
/***************************************************/
void
create_gcs (playerstruct *pl)
{
  int screen_depth;
  int screen_num;
  Colormap col;

  fprintf (stderr,"entered\n");
  screen_num = DefaultScreen (pl->display);
  screen_depth = DefaultDepth(pl->display, screen_num);
  
  pl->display_depth = screen_depth;
  if (screen_depth < 8)
    {
      pl->color = 0;
      return;
    }
  
  pl->color = 1;
  
  col = DefaultColormap(pl->display, screen_num);
  
  if (alloc_colors (pl,col))
    return;
  
  fprintf(stderr,"making my own!\n");
  col = XCreateColormap(pl->display, pl->main.window,
			DefaultVisual(pl->display, screen_num), AllocAll);
  
  if ( !store_colors (pl,col))
      pl->color = 0;
  fprintf (stderr,"exit\n");
}


/***************************************************/
/* create_bw                                       */
/*   simply copy the default white and black pixels*/
/* to all color pixel entries to create a b/w      */
/* color schema                                    */
/***************************************************/
void
create_bw(playerstruct *pl)
{
  int index;

  for (index =0 ; index < NUM_COLORS; index ++)
    {
      pl->pixels[index <<1] = pl->whitepix;
      pl->pixels[(index <<1)+1] = pl->blackpix;
    }
}

/***************************************************/
/* make_masks                                      */
/*   make the pixmaps which will serve as clipping */
/* masks for the objects                           */
/***************************************************/
void
make_masks(playerstruct *pl)
{
  int counter;
  char temp[1];

  pl->masks[0] = XCreatePixmap(pl->display,pl->main.window,
					 64,64,1);
  pl->bw_gc = XCreateGC (pl->display,pl->masks[0],GC_MASK,
			 init_gc_values(1,0,pl->itemfont->fid,GXcopy));

  for (counter = 1; counter < NUM_PICS; counter ++)
    {
      pl->masks[counter] = XCreatePixmap(pl->display,pl->main.window,
					 64,64,1);     
      XFillRectangle (pl->display,pl->masks[counter],pl->bw_gc,0,0,64,64);
      temp[0] = counter;
      XDrawImageString(pl->display,pl->masks[counter],pl->bw_gc,0,64,
			temp,1);
      pl->maskgcs[counter]=XCreateGC (pl->display,pl->viewpix.window,GC_MASK,
		 init_gc_values(pl->blackpix,pl->whitepix,
				pl->itemfont->fid,GXcopy));
      XSetClipMask(pl->display,pl->maskgcs[counter],pl->masks[counter]);
    }
  
  fprintf (stderr,"done....\n");
}
/***************************************************/
/* make_player_window                              */
/*   make all the neccessary calls to set up the   */
/* conection, drawing modes (GC's) and windows     */
/***************************************************/
void
make_player_window(playerstruct * pl)
{
  init_display(pl);
  load_fonts(pl);
  make_main_window(pl);
  create_gcs(pl);  
  set_back_pixmap(pl, &pl->main, MAINBITMAP);
  if (pl->color == 0)
    create_bw(pl);
  make_view_window(pl);
  make_viewpix(pl);
  make_itempix(pl);
  make_text_window(pl);
  make_stats_window(pl);
  make_item_window(pl);
  make_control_window(pl);
  make_cursors(pl);
  make_masks(pl);
  set_wm_icon(pl);
  XMapRaised(pl->display, pl->main.window);
  XMapSubwindows(pl->display, pl->main.window);
}



