/*********************************************************/
/* file: handle.c                                        */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 sept  93                                 */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  if you add place your name here or below             */
/*                                                       */
/* description:                                          */
/*  handle.c contains all the subroutine for the actual  */
/* object drivers                                        */
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <standard.h>

void
nullfunc()
{
}

void
/***************************************************************/
/*              player handle routines                         */
/***************************************************************/
player_turn(object * target)
{
  XEvent          event;

  playerstruct   *pl = OB_PLDATA(target);

  CLEAR_PL_HAS_MOVED(pl);
  CLEAR_PL_HAS_ATTACKED(pl);
  CLEAR_PL_HAS_USED(pl);

  while (XEventsQueued(pl->display, QueuedAfterReading))
  {
    XNextEvent(pl->display, &event);
    switch (event.type)
    {
      /* handle expose events */
    case (Expose):
      if (event.xexpose.window == pl->text.window)
        SET_PL_TEXT_WRITTEN(pl);
      else if (event.xexpose.window == pl->item.window)
        SET_PL_ITEM_CHANGED(pl);
      else if (event.xexpose.window == pl->controls.window)
        SET_PL_COMS_CHANGED(pl);
      else if (event.xexpose.window == pl->stats.window)
        SET_PL_STATS_CHANGED(pl);
      break;
      /* pass the button control to the current */
      /* button controller                      */
    case (ButtonPress):
      pl->button_control(pl, event);
      break;

      /*
       * pass the keypress to the current keypress
       * controlloer
       */
    case (KeyPress):
      pl->key_control(pl, event);
      break;
    default:
      break;
    }
  }
}

void
player_button(object * target, object * who, Window * window, int *button,
        int *xloc, int *yloc, void *data5)
{
  int             count;
  object         *tobj;
  char            tmpbuf[100];
  playerstruct   *pl = OB_PLDATA(target);

  switch (pl->obj->state.normal)
  {
  case (0):
    break;
  case (1):
    break;
  case (2):
    break;
  case (3):
    if (*window == pl->view.window)
      switch (*button)
      {
      case (Button1):
        move_player(pl, (pl->obj->facing + pl->dir) % 8);
        break;
      case (Button2):
        apply_bstate(pl, pl->dir);
        break;
      case (Button3):
        switch (pl->dir)
        {
        case (0):
          move_player(pl, target->facing);
          break;
        case (1):
        case (2):
        case (3):
          target->facing = (target->facing + 2) % 8;
          break;
        case (4):
          move_player(pl, (target->facing + 4) % 8);
          break;
        case (5):
        case (6):
        case (7):
          target->facing = (target->facing + 6) % 8;
          break;
        default:
          fprintf(stderr, "direction range error in character");
          break;
        }
        break;

      default:
        write_text(OB_PLDATA(target), "How Many God damn mouse buttons do you have!");
      }
    else if (*window == pl->controls.window)
    {
      count = ((*yloc - 2) / (pl->th + 2));
      SET_PL_COMS_CHANGED(pl);
      switch (count)
      {
      case (1):
        pl->buttonstate = 1;
        break;
      case (2):
        pl->buttonstate = 2;
        break;
      default:
        fprintf(stderr, "unknown button state \n");
        break;
      }
    } else if (*window == pl->item.window)
    {
      if ((*yloc - 2) < pl->th)
      {
        count = 4 * (*xloc) / (ITEM_W);
        switch (count)
        {
        case (0):
          shift_items_up(pl);
          break;
        case (2):
          shift_items_left(pl);
          break;
        case (1):
          shift_items_right(pl);
          break;
        case (3):
          shift_items_down(pl);
          break;
        }
      } else
      {
        tobj = get_selected_item(pl, *xloc, *yloc);
        if (tobj != NULL)
        {
          sprintf(tmpbuf, "Name: %s  Weight:%d", tobj->name, GET_WEIGHT(tobj));
          write_text(pl, "item selected");
          write_text(pl, tmpbuf);
        } else
          write_text(pl, "Nothing There");
      }
    }
    break;
  default:
    break;
  }
}

void
player_key(object * target, object * who, char *data)
{
  playerstruct   *pl;
  pl = OB_PLDATA(target);

  switch (pl->obj->state.normal)
  {
  case (0):
    if (strlen(pl->parsing) > 0)
    {
      /* truncate name */
      pl->parsing[14] = '\0';
      SET_NAME(pl->obj, pl->parsing);
      SET_PL_STATS_CHANGED(pl);
      pl->obj->state.normal = 3;
      reset_normal_controllers(pl);
    }
    break;
  case (1):
    /* reserved for password */
    break;
  case (2):
    /* reserverd for race selection */
    break;
  case (3):
    switch (pl->keystate)
    {
    case (PARSE_STATE):
      parse_command(pl);
      pl->keystate = KEY_COMM_STATE;
      pl->key_control = (&get_single_key);
      break;
    case (KEY_COMM_STATE):
      decode_keycommand(pl, (KeySym *) data);
      break;
    }
    break;
  default:
    break;
  }
}

void
player_attack(object * target, object * hitter, int damage, int type)
{
  char            tmpbuf[100];
  if (damage > 0)

    sprintf(tmpbuf, "%s hits you for %d points!", GET_NAME(hitter), damage);

  else
    sprintf(tmpbuf, "%s tries to hit you but misses!", GET_NAME(hitter));
  write_text(OB_PLDATA(target), tmpbuf);

}

/*****************************************************************/
/*         enemy handle routines                                 */
/*****************************************************************/
void
enemy_turn(object * obj)
{

  object         *temp = find_nearest_player(obj);
  int             dir = find_direction(obj, temp);

  switch (obj->state.normal)
  {
  case (0):
    if (temp == NULL)
    {
      if (!move_object(obj, obj->facing))
        obj->facing = random() % 8;
    } else if (!move_object(obj, dir))
    {
      if (distance(temp, obj) > 2)
        move_object(obj, random() % 8);
      else
        attack_object(temp, obj, NULL);
    }
    break;
  case (1):
    if (OB_ATTR(obj).phys == OB_ATTR(obj).physmax)
      obj->state.normal = 0;
    else if (!move_object(obj, (dir + 4) % 8))
      move_object(obj, (dir + random() % 5 + 2) % 8);
    break;

  default:
    obj->state.normal = 0;
    break;
  }
}

void
enemy_button()
{
}

void
enemy_attack(object * target, object * hitter, int damage)
{
  if (IS_PLAYER(hitter))
    write_text(OB_PLDATA(hitter), "its eye bulges out!");
  if (OB_ATTR(target).phys < (OB_ATTR(target).physmax / 2))
    target->state.normal = 1;
}

void
enemy_key()
{
}

/*****************************************************************/
/*         george handle routines                                */
/*****************************************************************/
void
george_turn(object * obj)
{
  if ((obj->facing > 7))
    obj->facing = random() % 8;
  if (!move_object(obj, obj->facing))
    obj->facing = random() % 8;
}

void
george_button()
{
}
void
george_key()
{
}

/*****************************************************************/
/*         NULL handle routines                                  */
/*****************************************************************/

void
null_turn(object * obj)
{
}

void
null_button(object * obj)
{

}

void
null_key(object * obj)
{
}

/*****************************************************************/
/*         tele1 handle routines                                 */
/*****************************************************************/
void
tele1_key(object * target, object * who, char *data)
{
  switch (target->state.input)
  {
    case (0):
    if (*(int *) data == 'y')
      if (target->data.tecomp->mapto[0] == '\0')
        write_text(OB_PLDATA(who), "this teleportation machine is broken!");
      else if (!send_to_map(who, target->data.tecomp->mapto, target->data.tecomp->x,
          target->data.tecomp->y, target->data.tecomp->z))
        write_text(OB_PLDATA(who),
             "there is something seriously wrong with space and time");
      else
      {
        write_text(OB_PLDATA(who),
            "You disappear for a brief moment....");
        write_text(OB_PLDATA(who),
          "and reappear somewhere else.....");
      }
    else
      write_text(OB_PLDATA(who), "alright then go away or something");
    break;
  default:
    fprintf(stderr, "tele 1 input driver error!");
    break;
  }
  reset_normal_controllers(OB_PLDATA(who));
}

void
tele1_interact(object * target, object * who)
{
  if (IS_PLAYER(who))
  {
    write_text(OB_PLDATA(who), "This is a teleporter");
    if (target->data.tecomp->mapto[0] != '\0')
    {
      write_text(OB_PLDATA(who), "It will take you to :");
      write_text(OB_PLDATA(who), target->data.tecomp->mapto);
    } else
    {
      write_text(OB_PLDATA(who), "it is OUT OF SERVICE");
      return;
    }
    write_text(OB_PLDATA(who), "Would you like to activate this machine?");
    OB_PLDATA(who)->key_object = target;
    OB_PLDATA(who)->key_control = (&get_single_key);
    OB_PLDATA(who)->button_control = (&get_no_buttonpress);
  }
}

/*****************************************************************/
/*       generic handle routines                                 */
/*****************************************************************/
void
generic_attack(object * target, object * hitter, int damage, int type)
{
  if (IS_PLAYER(hitter))
    write_text(OB_PLDATA(hitter), "it screams!");
}

void
unfriendly_interact(object * target, object * who)
{
  int             k;
  if (IS_PLAYER(who))
  {
    k = random() % 4;
    switch (k)
    {
    case (0):
      write_text(OB_PLDATA(who), "it has a look that says leave me alone!");
      break;
    case (1):
      write_text(OB_PLDATA(who), "it glares back at you!");
      break;
    case (2):
      write_text(OB_PLDATA(who), "it growls!");
    case (3):
      write_text(OB_PLDATA(who), "looks like it eats anything that moves!");
      break;
    }
  }
}

void
dumb_interact(object * target, object * who)
{
  int             k;
  if (IS_PLAYER(who))
  {
    k = random() % 3;
    switch (k)
    {
    case (0):
      write_text(OB_PLDATA(who), "it has no real mind of its own.");
      break;
    case (1):
      write_text(OB_PLDATA(who), "if it could, it would.");
      break;
    case (2):
      write_text(OB_PLDATA(who), "it sits there dumbly!");
      break;
    }
  }
}

/**********************************************/
/* tele_floor drivers                         */
/**********************************************/
void
tele_turn(object * ob)
{
  if (ob->data.tecomp->mapto != NULL && ob->above != NULL && !IS_DOOR(ob->above))
  {
    if (IS_PLAYER(ob->above) && PL_HAS_DM_POWER(OB_PLDATA(ob->above)))
      return;
    send_to_map(ob->above, ob->data.tecomp->mapto, ob->data.tecomp->x,
          ob->data.tecomp->y, ob->data.tecomp->z);
  }
}

/***********************************************/
/* melee_weapon drivers                        */
/***********************************************/
void
melee_weap_use(object * target, object * ob, object * ob2, int dir)
{
  if (ob2 != NULL)
    attack_object(ob2, ob, target);
  /* how do you use a melee weapon in a direction? */
  else if (IS_PLAYER(ob))
    write_text(OB_PLDATA(ob), "you must specify an object to use this on");
}

/***********************************************/
/* plain door_drivers                          */
/***********************************************/
void
door_interact(object * target, object * who)
{
  object         *curr,
                 *tmp;
  if (IS_ALIVE(who))
  {
    /* state 0 = closed  1= open */
    switch (target->state.normal)
    {
    case (0):
      CLEAR_BLOCKS_MOVE(target);
      CLEAR_BLOCKS_VIEW(target);
      for (curr = target; curr != NULL; curr = curr->link)
        SET_PIC(curr, 32);
      target->state.normal = 1;
      break;
    case (1):
      SET_BLOCKS_MOVE(target);
      SET_BLOCKS_VIEW(target);
      for (curr = target, tmp = target->gen_ob; curr != NULL;
           curr = curr->link, tmp = tmp->link)
        SET_PIC(curr, GET_PIC(tmp));
      target->state.normal = 0;
      break;
    default:
      fprintf(stderr, "door driver error");
      target->state.normal = 1;
      break;
    }
  }
}

/***************************************/
/* lock_door driver                    */
/***************************************/
void
lock_door_interact(object * target, object * who)
{
  object         *curr,
                 *tmp;
  if (IS_ALIVE(who))
  {
    /* state 0 = closed  1= open */
    switch (target->state.normal)
    {
    case (0):
      switch (target->state.input)
      {
        /* state 0 == locked */
      case (0):
        if (IS_PLAYER(who))
          write_text(OB_PLDATA(who), "The door is locked!");
        break;
      case (1):
        CLEAR_BLOCKS_MOVE(target);
        CLEAR_BLOCKS_VIEW(target);
        for (curr = target; curr != NULL; curr = curr->link)
          SET_PIC(curr, 32);
        target->state.normal = 1;
        break;
      }
      break;
    case (1):
      SET_BLOCKS_MOVE(target);
      SET_BLOCKS_VIEW(target);
      for (curr = target, tmp = target->gen_ob; curr != NULL;
           curr = curr->link, tmp = tmp->link)
        SET_PIC(curr, GET_PIC(tmp));
      target->state.normal = 0;
      break;
    default:
      fprintf(stderr, "door driver error");
      target->state.normal = 1;
      break;
    }
  } else
    /* key == 0 */
  if (who->subtype == 0)
  {
    if (who->state.normal == target->data.docomp->keynum)
    {
      if (who->owner != NULL && IS_PLAYER(who->owner))
        write_text(OB_PLDATA(who->owner), "You Hear a soft clicking sound");
      switch (target->state.input)
      {
      case (0):
        target->state.input = 1;
        break;
      case (1):
        target->state.input = 0;
        break;
      }
    } else if (who->owner != NULL && IS_PLAYER(who->owner))
      write_text(OB_PLDATA(who->owner), "You hear a loud CLACK!");
  }
}


void
key_use(object * target, object * ob, object * ob2, object * dir)
{
  if (ob2 == NULL)
    ob2 = MAP_ADJ_OBJ(ob, dir);

  ob2 = HEAD(ob2);

  SENDRQ(ob2, interact) (ob2, target);
}

/************************************/
/* acid driver                      */
/************************************/
void
acid_turn(object * ob)
{
  if (IS_REMOVED(ob))
    return;
  if (ob->above != NULL)
    damage_object(ob->above, ob, GET_PHYS(ob), acid);
  if (ob->below != NULL)
    damage_object(ob->below, ob, GET_PHYS(ob), acid);
  ob->state.normal--;
  if (ob->state.normal == 0)
  {
    remove_ob(ob);
    free_object(ob);
  }
}

/************************************/
/* fire driver                      */
/************************************/
void
fire_turn(object * ob)
{
  if (IS_REMOVED(ob))
    return;
  if (ob->above != NULL)
    damage_object(ob->above, ob, GET_PHYS(ob), heat);
  if (ob->below != NULL)
    damage_object(ob->below, ob, GET_PHYS(ob), heat);
  ob->state.normal--;
  ob->facing = (ob->facing + 1) % 8;
  if (ob->state.normal == 0)
  {
    remove_ob(ob);
    free_object(ob);
  }
}

void
forward_fire_turn(object * ob)
{
  object         *tobj;
  
  if (IS_REMOVED(ob))
    return;
  if (ob->above != NULL)
    damage_object(ob->above, ob, GET_PHYS(ob), heat);
  if (ob->below != NULL)
    damage_object(ob->below, ob, GET_PHYS(ob), heat);
  ob->facing = (ob->facing + 1) % 4;
  if (ob->state.aux == 0)
  {
    if (IN_MAP(ob->mapin, ADJ_X(ob, ob->state.input), ADJ_Y(ob, ob->state.input)) &&
        !MAP_MOVE_BLOCKED(ob->mapin, ADJ_X(ob, ob->state.input),
              ADJ_Y(ob, ob->state.input), ob->z))
    {
      tobj = get_general_object("forward_fire");
      tobj->state.input = ob->state.input;
      insert_ob_in_map(ob->mapin, tobj, ob->x + xdeltas[ob->state.input],
           ob->y + ydeltas[ob->state.input],
           ob->z);
      tobj->state.normal = ob->state.normal;
      if (tobj->mapin == NULL)
        free_object(tobj);
      ob->state.aux = 1;
    } else if (IN_MAP(ob->mapin, ADJ_X(ob, ob->state.input),
          ADJ_Y(ob, ob->state.input)))
    {
      damage_object(MAP_ADJ_OBJ(ob, ob->state.input), ob, GET_PHYS(ob), heat);
    }
  }
  ob->state.normal--;
  if (ob->state.normal == 0)
  {
    remove_ob(ob);
    free_object(ob);
  }
}


/************************************/
/* fire_floor driver                */
/************************************/
void
fire_floor_turn(object * ob)
{
  object         *tobj;
  if (ob->state.normal == 0)
  {
    if (rand() % 8 == 0)
    {
      tobj = get_general_object("fire");
      tobj->state.normal = random() % 10 + 1;
      insert_ob_in_map(ob->mapin, tobj, ob->x, ob->y, ob->z);
      ob->state.normal = tobj->state.normal;
    }
  } else
    ob->state.normal--;
}

/************************************/
/* gell drivers                     */
/************************************/
void
gell_turn(object * obj)
{

  if ((obj->facing > 7))
    obj->facing = random() % 8;
  if (!IS_REMOVED(obj) && move_object(obj, obj->facing))
    insert_ob_under_map(obj->mapin, get_general_object("acid"),
			obj->x, obj->y, obj->z);
  else
    obj->facing = random() % 8;
}

void
gell_attack(object * obj, object * hitter, int damage, int type)
{
  int             i;
  if (IS_PLAYER(hitter))
    write_text(OB_PLDATA(hitter), "Splishhhh!");
  if (strcmp(GET_NAME(hitter), "acid") && type == physical)
    for (i = 0; i < 8; i++)
      insert_ob_under_map(obj->mapin, get_general_object("acid"), obj->x + xdeltas[i],
              obj->y + ydeltas[i], obj->z);
}

/****************************************/
/* merc drivers                         */
/****************************************/
void
merc_attack(object * target, object * hitter, int damage, int type)
{
  if (IS_PLAYER(hitter))
    write_text(OB_PLDATA(hitter), "He Howls!");
  if (IS_ALIVE(hitter))
  {
    target->data.ancomp->interest = hitter;
    if (GET_PHYS(target) < (GET_PHYSMAX(target) / 10))
      target->state.normal = 2;
    else
      target->state.normal = 1;
  }
}

void message_interact(object *target,object *who)
{
  char tmp_string[255];

  if(IS_PLAYER(who))
    {
      if ( target->message != NULL)
	sprintf (tmp_string,"%s says:%s",target->name,target->message->text);
      else
	sprintf (tmp_string,"%s mumbles something incomprehensible",
		 target->name);
      
      write_text (OB_PLDATA(who),tmp_string);
    }
}
void
merc_turn(object * obj)
{
  object         *tmp = obj->data.ancomp->interest;
  int             tval;
  switch (obj->state.normal)
  {
  case (0):
    if ((obj->facing > 7))
      obj->facing = random() % 8;
    if (!(random() % 3) && !move_object(obj, obj->facing))
      obj->facing = random() % 8;
    break;
  case (1):
    if (tmp == NULL || OB_IS_FREED(tmp)
        || IS_REMOVED(tmp) || tmp->mapin != obj->mapin)
    {
      obj->data.ancomp->interest = NULL;
      obj->state.normal = 0;
    } else
    {
      tval = find_direction(obj, tmp);
      if (!move_object(obj, tval))
        if (distance(obj, tmp) > 2)
          move_object(obj, random() % 8);
        else
          attack_object(tmp, obj, AN_OB_ITEM(obj).held);
    }
    break;
  case (2):
    if ((GET_PHYS(obj) > (GET_PHYS(obj) / 2)) || tmp == NULL || OB_IS_FREED(tmp)
        || OB_IS_REMOVED(tmp) || tmp->mapin != obj->mapin)
      obj->state.normal = 1;
    else
    {
      tval = (find_direction(obj, tmp) + 4) % 8;
      if (!move_object(obj, tval))
        move_object(obj, (tval + 6 + random() % 5) % 8);
    }
    break;
  default:
    obj->state.normal = 0;
    break;
  }
}

/********************************/
/* flamer_interact              */
/********************************/
void
gun_interact(object * ob, object * interactor)
{
  object         *tobj;
  if (ob->owner != interactor)
    return;

  if (ob->holding != NULL)
  {
    tobj = ob->holding;
    remove_held(tobj);
    add_holding(interactor, tobj);
    if (IS_PLAYER(interactor))
      write_text(OB_PLDATA(interactor), "you unload the bg101");
  } else
  {
    for (tobj = interactor->holding; tobj != NULL; tobj = tobj->below)
    {
      if (tobj->type == 3 && tobj->subtype == 9)
        break;
    }
    if (tobj == NULL)
    {
      if (IS_PLAYER(interactor))
        write_text(OB_PLDATA(interactor), "you have no pyro cartridge to load");
      return;
    }
    remove_held(tobj);
    add_holding(ob, tobj);
    if (IS_PLAYER(interactor))
      write_text(OB_PLDATA(interactor), "you load the bg101 with a pyrocart");
  }
  if (IS_PLAYER(interactor))
    SET_PL_ITEM_CHANGED(OB_PLDATA(interactor));
}

void
gun_use(object * target, object * ob, object * ob2, int dir)
{
  object         *tobj;
  if (ob != target->owner)
    return;
  if (target->holding == NULL || target->holding->state.normal == 0)
  {
    if (IS_PLAYER(ob))
      write_text(OB_PLDATA(ob), "pufffff...");
    return;

  }
  SENDRQ(target->holding, interact) (target->holding, target);
  tobj = get_general_object("forward_fire");
  tobj->state.normal = 5;
  tobj->state.input = ob->facing;
  insert_ob_in_map(ob->mapin, tobj, ob->x + xdeltas[ob->facing],
       ob->y + ydeltas[ob->facing],
       ob->z);

}

void
pyro_cart_attack(object * target, object * hitter, int damage, int type)
{
  map            *tmap;

  if ((type == physical || type == heat || type == electric) &&
      target->owner == NULL)
  {
    if (target->mapin == NULL)
      if (target->owner == NULL)
        return;
      else
      {
        if ((tmap = target->owner->mapin) == NULL)
          return;
      }
    else
      tmap = target->mapin;

    cause_explosion (target,tmap,target->x,target->y,
		     target->z,1);
    remove_ob(target);
    free_object(target);
  }
}
void
pyro_cart_interact(object * ob, object * who)
{
  char tstring[64];
  if (IS_PLAYER(who))
  {
    if (ob->state.normal >0)
    {
      sprintf (tstring,"Looks like you could squeeze off %d shots using it",
        ob->state.normal);
      write_text(OB_PLDATA(who),tstring);
    }
    else
      write_text(OB_PLDATA(who),"You shake it, it sounds empty");
  }
  else
  {
  if (!IS_WEAPON(who) || who->subtype != 1 || who != ob->owner)
    return;
  else
    ob->state.normal--;
  }
}

void
throw_turn(object * ob)
{
  int             k;
  map            *tmap;
  if (ob->owner == NULL || IS_REMOVED(ob->owner))
  {
    if (ob->owner != NULL)
      remove_held(ob);
    else
      remove_ob(ob);
    free_object(ob);
    return;
  }
  if (!move_object(ob->owner, ob->state.input))
  {
    damage_object(TOP_MAP_OBJ(ob->owner->mapin,
          ob->owner->x, ob->owner->y, ob->owner->z),
       ob->owner, GET_WEIGHT(ob->owner) / 1000, physical);
    for (k = ob->owner->z - 1; 
	 k >= 0 && 
	 !MAP_MOVE_BLOCKED(ob->owner->mapin, ob->owner->x, ob->owner->y, k);
	 k--);
    if (k >= 0)
      damage_object(TOP_MAP_OBJ(ob->owner->mapin,ob->owner->x, ob->owner->y, k)
		    , ob->owner, GET_WEIGHT(ob->owner) / 100, physical);
    tmap = ob->owner->mapin;
    remove_ob(ob->owner);
    insert_ob_in_map(tmap, ob->owner, ob->owner->x, ob->owner->y, 0);
    remove_held(ob);
    free_object(ob);
    return;
  }
  ob->state.normal--;
  if (ob->state.normal == 0)
  {
    for (k = ob->owner->z - 1; 
	 k >= 0
	 && !MAP_MOVE_BLOCKED(ob->owner->mapin, ob->owner->x,
			      ob->owner->y, k);
	 k--);

    if (k >= 0)
      damage_object(TOP_MAP_OBJ(ob->owner->mapin,
               ob->owner->x, ob->owner->y, k),
        ob->owner, GET_WEIGHT(ob->owner) / 100, physical);

    tmap = ob->owner->mapin;
    remove_ob(ob->owner);
    insert_ob_in_map(tmap, ob->owner, ob->owner->x, ob->owner->y, 0);
    remove_held(ob);
    free_object(ob);
    return;
  }
}

void
expl_force_turn(object * ob)
{
  object         *tobj;

  if (IS_REMOVED(ob))
  {
    free_object(ob);
    return;
  }
   
  for (tobj = TOP_MAP_OBJ(ob->mapin, ob->x, ob->y, ob->z);
       tobj != NULL; tobj = tobj->below)
    if (ob != tobj)
    {
      damage_object(tobj, ob, GET_PHYS(ob), physical);
      damage_object(tobj, ob, GET_PHYS(ob), heat);
    }

  ob->facing = (ob->facing +1)%8;
  if (ob->state.normal -- == 0)
  {
    remove_ob(ob);
    free_object(ob);
  }
}

void
bomb_turn(object * ob)
{
  object        *tobj2;
  switch (ob->state.input)
  {
  case (0): /* bomb is off */
    return;
    break;
  case (1): /* bomb in on and counting down */
    if (ob->state.normal-- ==0)
      {
	ob->state.input = 2;
	SET_PIC (ob,32);
      }
    break;
  case(2): /* bomb has exploded! */
    if (IS_REMOVED(ob))
      {
        if (ob->owner == NULL)
          return;
	
        for (tobj2 = ob->owner; tobj2->owner != NULL; tobj2 = tobj2->owner)
	  {
	    damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			  heat);
	    damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			  physical);
	  }
      } 
    else
      tobj2 = ob;
    
    cause_explosion (ob,tobj2->mapin,tobj2->x,tobj2->y,
		     tobj2->z,ob->state.aux);
    /*
    if (tobj2 == ob)
    for (temp_x = tobj2->x - ob->state.aux; temp_x <= tobj2->x + ob->state.aux;
	 temp_x ++)
      for (temp_y = tobj2->y - ob->state.aux;
	   temp_y <= tobj2->y + ob->state.aux; temp_y ++)
	if(IN_MAP(tobj2->mapin,temp_x,temp_y))
	  {
	  if(MAP_MOVE_BLOCKED(tobj2->mapin,temp_x,temp_y,tobj2->z))
	    {
	      damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			    heat);
	      damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			    physical);
	    }
	 
	  tobj1 = get_general_object("fire");
	  tobj1->state.normal = ((3*ob->state.aux)-abs(tobj2->x-temp_x)
	    -abs(tobj2->y-temp_y))*2;
	  SET_PHYS(tobj1,5* (3*ob->state.aux)-abs(tobj2->x-temp_x)
	    -abs(tobj2->y-temp_y));
	  insert_ob_in_map (ob->mapin,tobj1,temp_x,temp_y,tobj2->z);

	  tobj1 = get_general_object("expl_force");
	  tobj1->state.normal = (3*ob->state.aux)-abs(tobj2->x-temp_x)
	    -abs(tobj2->y-temp_y);
	  SET_PHYS(tobj1,5* (3*ob->state.aux)-abs(tobj2->x-temp_x)
	    -abs(tobj2->y-temp_y));
	  insert_ob_in_map (ob->mapin,tobj1,temp_x,temp_y,tobj2->z);
	}*/
    remove_ob (ob);
    free_object (ob);
    break;
  default:
    ob->state.normal = 0;
    ob->state.input = 1;
    break;
  }
}

void
explosion_turn(object *ob)
{
  object         *tobj1,
                 *tobj2;
  int            temp_x,temp_y;

   if (IS_REMOVED(ob))
      {
        if (ob->owner == NULL)
          return;
	
        for (tobj2 = ob->owner; tobj2->owner != NULL; tobj2 = tobj2->owner)
	  {
	    damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			  heat);
	    damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			  physical);
	  }
      } 
    else
      tobj2 = ob;

    if (tobj2 == ob)
    for (temp_x = tobj2->x - ob->state.aux; temp_x <= tobj2->x + ob->state.aux;
	 temp_x ++)
      for (temp_y = tobj2->y - ob->state.aux;
	   temp_y <= tobj2->y + ob->state.aux; temp_y ++)
	if(IN_MAP(tobj2->mapin,temp_x,temp_y))
	  {
	  if(MAP_MOVE_BLOCKED(tobj2->mapin,temp_x,temp_y,tobj2->z))
	    {
	      damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			    heat);
	      damage_object(tobj2, ob,(ob->state.aux+1)*(ob->state.aux+1),
			    physical);
	    }
	  else
	    {
	      tobj1 = get_general_object("expl_force");
	      tobj1->state.normal = (2*ob->state.aux)-abs(tobj2->x-temp_x)
		-abs(tobj2->y-temp_y);
	      SET_PHYS(tobj1,(2*ob->state.aux)-abs(tobj2->x-temp_x)
		-abs(tobj2->y-temp_y));
	      insert_ob_in_map (ob->mapin,tobj1,temp_x,temp_y,tobj2->z);
	    }
	}
}
 
void
bomb_attack(object * target, object * hitter, int damage, int type)
{
  if ((type == physical || type == heat || type == electric) &&
      target->owner == NULL)
    {
      cause_explosion (target, target->mapin, target->x, target->y,
		       target->z, target->state.aux);
      remove_ob (target);
      free_object (target);
    }
}

void
bomb_use (object *target,object *ob,object *ob2,int dir)
{
  if (IS_PLAYER(ob))
  {
    write_text (OB_PLDATA(ob),"You Hear a Ticking Sound...");
    write_text (OB_PLDATA(ob),"You See a red light flashing on the bomb...");
  }
  target->state.input = 1;
}

/* the switch sends out a simple signal that will notify objects on */
/* its channels that the object has reversed modes                  */
void
switch_interact (object *target, object * who)
{
  if (target->facing != ((who->facing +4)%8))
    return;

  /* is this really neccesary ??? */
  target->state.normal = !target->state.normal;
  
  send_all_channels (target,0,NULL);
  
  if (IS_PLAYER (who))
    {
      write_text (OB_PLDATA(who),"You press the Switch...");
      write_text (OB_PLDATA(who),"You hear a clicking noise");
    }
}

void
switch_door_channel(object * target,object * sender, int data1,void *data2)
{
  object *curr,*gen;
  
  switch (target->state.normal)
    {
    case(0): /* closed - so set to open */
      CLEAR_BLOCKS_MOVE(target);
      CLEAR_BLOCKS_VIEW(target);
      
      for (curr = target; curr != NULL; curr = curr->link)
	SET_PIC(curr,32);
      
      target->state.normal = 1;
      break;
    
    case(1): /* open - so set to closed */
      SET_BLOCKS_MOVE(target);
      SET_BLOCKS_VIEW(target);
      for (curr = target, gen = target->gen_ob; curr != NULL;
	   curr = curr->link, gen = gen->link)
	SET_PIC(curr,GET_PIC (gen));
      target->state.normal = 0;
      break;
      
    default:
      fprintf (stderr,"error in switch door driver");
      break;
    }
}

/* this is included here to avoid lengthly externs... */
#include <handle.h>





