/*********************************************************/
/* file: resdata.c                                       */
/*                                                       */
/* Copyright (C) 1994 Karl A Holland                     */
/*                                                       */
/* author: Karl A Holland                                */
/*                                                       */
/* created:   1 sept 93                                  */
/*                                                       */
/* modified:  7 february 94                              */
/*                                                       */
/* contributing authors:                                 */
/*  if you add place your name here or below             */
/*                                                       */
/* description:                                          */
/*   resdata.c is one of the files for resedit, the      */
/* object resource editor for grudge. resdata.c          */
/* encapsulates all of the data handling of objects that */
/* is not taken care of in obdata.c for the resource     */
/* editor                                                */ 
/*                                                       */
/* LICENSING:                                            */
/*   a full license  is provided in the file LICENSE     */
/* which should have been included with this program when*/
/* it was copied. It may be found in the top directory   */
/* of any NORMAL distribution, named LICENSE             */
/*********************************************************/
#include <resedit.h>

 
void
apply_inorder(tree * tree_loc, void (*func) (tree *))
{
  if (tree_loc != NULL)
  {
    apply_inorder(tree_loc->left, func);
    func(tree_loc);
    apply_inorder(tree_loc->right, func);
  }
}

void
apply_preorder(tree * tree_loc, void (*func) (tree *))
{
  if (tree_loc != NULL)
  {
    func(tree_loc);
    apply_inorder(tree_loc->left, func);
    apply_inorder(tree_loc->right, func);
  }
}

void
increment_position(tree * atree)
{
  atree->position++;
}

void
decrement_position(tree * atree)
{
  atree->position--;
}

tree          **
rightmostchild(tree ** atree)
{
  if (*atree != NULL)
    while ((*atree)->right != NULL)
      atree = &((*atree)->right);
  else
    return NULL;
  return atree;
}

tree          **
leftmostchild(tree ** atree)
{
  if (*atree != NULL)
    while ((*atree)->left != NULL)
      atree = &((*atree)->left);
  else
    return NULL;
  return atree;
}

object *
get_object_from_pos (tree *atree,int pos)
{
	if (atree == NULL)
  	return NULL;
  while (atree != NULL && atree->position != pos)
    {
       if (atree->position >  pos)
     		atree = atree->left;
      else
        atree = atree->right;
    }
 return (atree == NULL ? NULL: atree->object);
}

int
insert_object(tree ** atree, object * obj)
{
  int             lexval,
                  lastright = -1;
                  
 
  if (*atree == NULL)
  {
    *atree = (tree *) malloc(sizeof(tree));
    (*atree)->object = obj;
    (*atree)->position = 0;
    (*atree)->left = (*atree)->right = NULL;
  } else
  {
    while (*atree != NULL)
    {
      lexval = strcmp(obj->name, (*atree)->object->name);
      if (lexval <= 0)
      {
        apply_inorder((*atree)->right, (&increment_position));
        increment_position(*atree);
        atree = &((*atree)->left);
      } else
      {
        lastright = (*atree)->position;
        atree = &((*atree)->right);
      }
    }
    *atree = (tree *) malloc(sizeof(tree));
    (*atree)->object = obj;
    (*atree)->position = lastright + 1;
    (*atree)->left = (*atree)->right = NULL;
    return (*atree)->position;
  }
  return (0);
}

void
delete_name(tree ** atree, char *name)
{
  int             lexval;
  tree          **rightchild;
  if (*atree == NULL)
  {
    return;
  } else
  {
    while (strcmp(name, (*atree)->object->name))
    {
      lexval = strcmp(name, (*atree)->object->name);
      if (lexval <= 0)
      {
        apply_inorder((*atree)->right, &decrement_position);
        decrement_position(*atree);
        atree = &((*atree)->left);
      } else
        atree = &((*atree)->right);
    }
    if (!strcmp(name, (*atree)->object->name))
    {
      apply_inorder((*atree)->right, &(decrement_position));
      rightchild = rightmostchild(&(*atree)->left);
      if (rightchild != NULL)
      {
        (*atree)->object = (*rightchild)->object;
        (*atree)->position = (*rightchild)->position;
        atree = &(*rightchild)->left;
        free(*rightchild);
        *rightchild = *atree;
      } else
      {
        rightchild = &((*atree)->right);
        if (*rightchild != NULL)
        {
          free(*atree);
          *atree = *rightchild;
        } else
        {
          free(*atree);
          *atree = NULL;
        }
      }
    }
  }
}

int
read_objects_in()
{
  FILE           *datafile;
  char            strdata[255];
  object         *tmpob;
  int             done = 0,
                  data = 1000;

  fprintf(stderr, "starting object read\n");
  strcpy(strdata, LIBDIR);
  strcat(strdata,"/");
  strcat(strdata, OBJFILE);
  if ((datafile = fopen(strdata, "r")))
  {
    fscanf(datafile, "%d", &data);
    fprintf(stderr, "%d\n", data);
    if (data != RD_START)
    {
      fprintf(stderr, "bad object file \n");
      fclose(datafile);
      return 1;
    } else
    {
      fprintf(stderr, "reading datafile \n");
      while (!done)
      {
        tmpob = readobject(datafile, MODE_GEN);
        switch (tmpob->type)
        {
        case (BAD):
          fprintf(stderr, "bad object in file\n");
          fclose(datafile);
          free_object(tmpob);
          return 1;
          break;
        case (EOobF):
          fprintf(stderr, "reached EOF in object\n");
          free_object(tmpob);
          break;
        case (FINISHED):
          free_object(tmpob);
          done = 1;
          break;
        default:
          /*fprintf(stderr, "name:%-20s\t hashlock:%5d\n",
          tmpob->name, obj_hash(tmpob->name));*/
          add_to_hashlist(tmpob);
          insert_object(&obj_tree, tmpob);
          break;
        }
      }
      if (!done)
      {
        fprintf(stderr, "reached EOF w/o finish\n");
        fclose(datafile);
        return 1;
      }
      fprintf(stderr, "completed reading object data\n");
    }
    fclose(datafile);
    return 0;
  }
  return 0;
}


void
insert_obs_in_list(Widget list)
{
  int             num_items,
                  loc = 0;
  tree          **temptree;
  XmString       *xmstrings;

  void            insert_in_loc(tree * atree)
  {
    /*fprintf(stderr, "inserting: %-20s\tNumber:%5d\n",
      atree->object->name, atree->position); */
    xmstrings[loc] = XmStringCreateSimple(atree->object->name);
    loc++;
  }
  temptree = rightmostchild(&obj_tree);
  num_items = (*temptree)->position;
  xmstrings = (XmString *) calloc(num_items+1, sizeof(XmString));
  apply_inorder(obj_tree, insert_in_loc);
  XmListAddItems(list, xmstrings, num_items, 1);
  for (loc = 0; loc < num_items; loc++)
    XmStringFree(xmstrings[loc]);
}

